//
//  FYAttributedMaker.h
//  YGRichTextDemo
//
//  Created by DaverZhou on 2020/8/27.
//  Copyright © 2020 DaverZhou. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

@interface FYAttributedMaker : NSObject
/// 初始化
/// @param string 字符串
- (instancetype)initWithString:(NSString *)string;
/// 富文本
- (NSMutableAttributedString *)result;

#pragma mark 链式调用后必须调用区间
/// 字体
- (FYAttributedMaker *(^)(UIFont *value))font;

/// 斜体
- (FYAttributedMaker *(^)(CGFloat value))italic;

/// 文字颜色
- (FYAttributedMaker *(^)(UIColor *value))foregroundColor;

/// 设置背景颜色
- (FYAttributedMaker *(^)(UIColor *value))backgroundColor;

/// 删除线高度
- (FYAttributedMaker *(^)(NSInteger value))strikethroughStyle;

/// 删除线颜色
- (FYAttributedMaker *(^)(UIColor *value))strikethroughColor;

/// 下滑线粗度
- (FYAttributedMaker *(^)(NSInteger value))underlineStyle;

/// 下滑线颜色
- (FYAttributedMaker *(^)(UIColor *value))underlineColor;

/// 字体描边宽度
- (FYAttributedMaker *(^)(CGFloat value))strokeWidth;

/// 字体描边颜
- (FYAttributedMaker *(^)(UIColor *value))strokeColor;

/// 字体阴影
- (FYAttributedMaker *(^)(NSShadow *value))shadow;

/// 字间距
- (FYAttributedMaker *(^)(CGFloat value))kern;

/// 行间距：同个range在调用range后，再设置lineBreakMode、textAlignment 会互相覆盖
- (FYAttributedMaker *(^)(CGFloat value))lineSpacing;

/// 对齐方式：同个range在调用range后，再设置lineBreakMode、lineSpacing 会互相覆盖
- (FYAttributedMaker *(^)(NSTextAlignment value))textAlignment;

/// 字符截断类型：同个range在调用range后，再设置textAlignment、lineSpacing 会互相覆盖
- (FYAttributedMaker *(^)(NSLineBreakMode value))lineBreakMode;

/// 设置URL跳转 UITextView才有效，UILabel和UITextField里面无效
- (FYAttributedMaker *(^)(NSString *value))link;

/// 插入图片(图片，尺寸，位置)
- (FYAttributedMaker *(^)(UIImage *image, CGRect bounds, NSInteger index))insertImage;

#pragma mark append string
/// 拼接string
- (FYAttributedMaker *(^)(NSString *string))appendString;

/// 合并
- (FYAttributedMaker *)mergeStrings;

#pragma mark 区间
/// 区间
- (FYAttributedMaker *(^)(NSUInteger loc, NSUInteger len))fy_inRange;
/// 区间
- (FYAttributedMaker *(^)(NSRange range))inRange;
/// 区间
- (FYAttributedMaker *(^)(void))allRange;

@end

NS_ASSUME_NONNULL_END
