//
//  FYCMacro.h
//  NBPlatformExample
//
//  Created by huangsy on 15/6/13.
//  Copyright (c) 2015年 stars. All rights reserved.
//
//  一些常用的宏定义


#define FYCoreName @"FYCore"
#define FYCoreResource @"FYCore"

#import "FYCore.h"
#import "UIScreen+FYC.h"
#import "UIView+FYC.h"
#import "UIColor+FYC.h"
#import "FYCConfigFileManager.h"
#import "FYCHint.h"

#define FYC_GAME_COMPANYID       [[FYCConfigFileManager sharedInstance] rowConfigWithKey:@"FY_GAME_COMPANYID"]
#define FYC_GAME_APPID           [[FYCConfigFileManager sharedInstance] rowConfigWithKey:@"FY_GAME_APPID"]
#define FYC_GAME_APPKEY          [[FYCConfigFileManager sharedInstance] configWithKey:@"FY_GAME_APPKEY"] // 已加密，需解密
#define FYC_GAME_ORIENTATION     [[FYCConfigFileManager sharedInstance] rowConfigWithKey:@"FY_GAME_ORIENTATION"]
#define FYC_GAME_DEBUG           [[FYCConfigFileManager sharedInstance] debugConfigWithKey:@"FY_GAME_DEBUG"]
#define FYC_GAME_CHANNELID       [[FYCConfigFileManager sharedInstance] rowConfigWithKey:@"FY_GAME_CHANNELID"]
#define FYC_GAME_EXTRA_JSON      [[FYCConfigFileManager sharedInstance] rowConfigWithKey:@"FY_GAME_EXTRA_JSON"]
#define FYC_GAME_ENVIRONMENT       [[FYCConfigFileManager sharedInstance] rowConfigWithKey:@"FY_GAME_ENVIRONMENT"]

// 判断iPad、iPhone
#define FYCIsPad ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad)
#define FYCIsPhone ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone)
#define FYCIsIPhoneX \
({BOOL isPhoneX = NO;\
if (@available(iOS 11.0, *)) {\
isPhoneX = [[UIApplication sharedApplication] delegate].window.safeAreaInsets.bottom > 0.0;\
}\
(isPhoneX);})

// 判断横竖屏
#define FYCIsLandscape ([UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeLeft || [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeRight)
#define FYCIsPortrait ([UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationPortrait || [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationPortraitUpsideDown)

// bundle 资源
#define FYCBundle(bundleName, fileName) [NSString stringWithFormat:@"%@.bundle/%@", bundleName, fileName]
#define FYCBundleImage(bundleName, fileName) [UIImage imageNamed:FYCBundle(bundleName, fileName)]

// 字符串、数组、字典、对象 判空
#define FYCStrClearNil(a) a == nil ? @"" : a
#define FYCStrEmpty(a) (a == nil || [a isEqual:[NSNull null]] || a.length == 0)
#define FYCArrEmpty(a) (a == nil || [a isEqual:[NSNull null]] || a.count == 0)
#define FYCDictEmpty(a) (a == nil || [a isEqual:[NSNull null]] || a.count == 0)
#define FYCObjNull(a) [a isEqual:[NSNull null]]
#define FYCTurnStr(a) [NSString stringWithFormat:@"%@", a?:@""]
#define FYCStr(format, ...) [NSString stringWithFormat:format, ##__VA_ARGS__]

// 屏幕宽、高
#define FYCScreenSize [UIScreen mainScreen].bounds.size
#define FYCScreenWidth [UIScreen mainScreen].bounds.size.width
#define FYCScreenHeight [UIScreen mainScreen].bounds.size.height
#define FYCScaleWidth(w) FYCScreenWidth / 375 * w
#define FYCScaleHeight(h) FYCScreenHeight / 667 * h

// hint hud
#define FYCHintShow(title) [[[FYCHint alloc] initWithTitle:title] show];

//
#define FYCClass NSStringFromClass([self class])
#define FYCMethod NSStringFromSelector(_cmd)

#define FYCLog(format, ...) do {\
    if ([FYC_GAME_DEBUG isEqualToString:@"1"] || FYCConfigFileManager.sharedInstance.getDebug) {\
        NSLog((@"#fy>>class:%@>>method:[%@]>>>>" format), FYCClass, FYCMethod, ##__VA_ARGS__);\
    }\
} while (0)

// 常量声明
#if !defined(FYC_CONST_STRING)
#   if defined(__cplusplus)
#       define FYC_CONST_STRING extern "C" NSString * const
#   else
#       define FYC_CONST_STRING NSString * const
#   endif
#endif

#if !defined(FYC_EXTERN_CONST_STRING)
#   if defined(__cplusplus)
#       define FYC_EXTERN_CONST_STRING extern "C" NSString * const
#   else
#       define FYC_EXTERN_CONST_STRING extern NSString * const
#   endif
#endif

// 弱引用 & 强引用
#define FYCWeakSelf(weakSelf)  __weak __typeof(&*self)weakSelf = self;
#define FYCStrongSelf(strongSelf) __strong __typeof(weakSelf)strongSelf = weakSelf;

#ifndef weakify
    #if defined(DEBUG)
        #if __has_feature(objc_arc)
        #define weakify(object) autoreleasepool{} __weak __typeof__(object) weak##_##object = object;
        #else
        #define weakify(object) autoreleasepool{} __block __typeof__(object) block##_##object = object;
        #endif
    #else
        #if __has_feature(objc_arc)
        #define weakify(object) try{} @finally{} {} __weak __typeof__(object) weak##_##object = object;
        #else
        #define weakify(object) try{} @finally{} {} __block __typeof__(object) block##_##object = object;
        #endif
    #endif
#endif

#ifndef strongify
    #if defined(DEBUG)
        #if __has_feature(objc_arc)
        #define strongify(object) autoreleasepool{} __typeof__(object) object = weak##_##object;
        #else
        #define strongify(object) autoreleasepool{} __typeof__(object) object = block##_##object;
        #endif
    #else
        #if __has_feature(objc_arc)
        #define strongify(object) try{} @finally{} __typeof__(object) object = weak##_##object;
        #else
        #define strongify(object) try{} @finally{} __typeof__(object) object = block##_##object;
        #endif
    #endif
#endif

/// 定义尺寸规则
/// iPad的尺寸为iPhone的1.5倍
#define FYCSizeRuler(s) FYCIsPad ? s * 1.5 : s

/////////////////////////////////////////////
// 以下部分后续会慢慢抛弃
#define FYCPageShowDuration 0.1
#define FYCPageHideDuration 0.2
#define FYCViewMoveDuration 0.3

#define FYCColorWhite       [UIColor colorWithHexString:@"FFFFFF" alpha:1.0]
#define FYCColorBlack       [UIColor colorWithHexString:@"000000" alpha:1.0]
#define FYCColorGary        [UIColor colorWithHexString:@"bfbfbf" alpha:1.0]
#define FYCColorLightGary   [UIColor colorWithHexString:@"c5c5c5" alpha:1.0]
#define FYCColorShadow      [UIColor colorWithHexString:@"000000" alpha:0.6]
#define FYCColorHintShadow  [UIColor colorWithHexString:@"000000" alpha:0.8]
#define FYCColorLightShadow [UIColor colorWithHexString:@"000000" alpha:0.2]
#define FYCColorRed         [UIColor colorWithHexString:@"F15757" alpha:1.0]
#define FYCColorGreen       [UIColor colorWithHexString:@"56B8B8" alpha:1.0]
#define FYCColorBlue        [UIColor colorWithHexString:@"9fb8e5" alpha:1.0]
#define FYCColorHex(hex)    [UIColor colorWithHexString:hex alpha:1.0]

#define FYCLableSize1 (FYCIsPad ? 11 : 10)
#define FYCLableSize2 (FYCIsPad ? 12 : 11)
#define FYCLableSize3 (FYCIsPad ? 13 : 12)
#define FYCLableSize4 (FYCIsPad ? 14 : 13)
#define FYCLableSize5 (FYCIsPad ? 15 : 14)
#define FYCLableSize6 (FYCIsPad ? 16 : 15)
#define FYCLableSize7 (FYCIsPad ? 17 : 16)
#define FYCLableSize8 (FYCIsPad ? 18 : 17)
#define FYCLableSize9 (FYCIsPad ? 19 : 18)
#define FYCLableSize10 (FYCIsPad ? 20 : 19)
#define FYCLableSize11 (FYCIsPad ? 21 : 20)
#define FYCLableSize12 (FYCIsPad ? 22 : 21)

#define FYCRulerScreenShortSide(scale) [UIScreen screenShortSide] * scale
#define FYCRulerScreenLongSide(scale) [UIScreen screenLongSide] * scale
#define FYCRulerScreenWidth(scale) [UIScreen screenWidth] * scale
#define FYCRulerScreenHeight(scale) [UIScreen screenHeight] * scale
#define FYCRulerViewWidth(view, scale) view.frameWidth * scale
#define FYCRulerViewHeight(view, scale) view.frameHeight * scale

#define FYCIsLessIos9 ([[[UIDevice currentDevice] systemVersion] floatValue] < 9.0)
#define FYCIsLessIos8 ([[[UIDevice currentDevice] systemVersion] floatValue] < 8.0)
#define FYCIsLessIos7 ([[[UIDevice currentDevice] systemVersion] floatValue] < 7.0)

#define FYCStatusBarHeight 20.0
#define FYCIPhoneXStatusBarHeight 44.0
