//
//  FYCOrderQueueManager.h
//  FYCoreDemo
//
//  Created by 吴垠锋 on 2018/11/15.
//  Copyright © 2018年 feiyu. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "FYCRequestQueueTask.h"

typedef void(^FYCRequestQueueResponse)(BOOL connected, NSString *result, NSDictionary *httpInfo, FYCRequestQueueTask *task);

/**
 网络请求串行队列,队列在执行期间要保证"该队列的对象不被销毁",该类最好不要创建局部对象
 */
@interface FYCRequestQueue : NSObject

@property (nonatomic, copy) FYCRequestQueueResponse response;

/**
 带标记的初始化方法,默认不加密

 @param flag 唯一标记,在存储时使用,如果不设置可能会造成 请求重复发送
 @return 返回对象
 */
- (instancetype)initWithFlag:(NSString*)flag;

/**
 带标记和加密的初始化方法

 @param flag 唯一标记,在存储时使用,如果不设置可能会造成 请求重复发送
 @param password 用于存储时加密
 @return 返回对象
 */
- (instancetype)initWithFlag:(NSString*)flag password:(NSString*)password;

/**
 带标记和请求成功回调的初始化方法,默认不加密

 @param flag 唯一标记,在存储时使用,如果不设置可能会造成 请求重复发送
 @param response 请求成功回调
 @return 返回对象
 */
- (instancetype)initWithFlag:(NSString*)flag response:(FYCRequestQueueResponse)response;

/**
 带标记和加密以及请求成功回调的初始化方法

 @param flag 唯一标记,在存储时使用,如果不设置可能会造成 请求重复发送
 @param password 用于存储时加密
 @param response 请求成功回调
 @return 返回对象
 */
- (instancetype)initWithFlag:(NSString*)flag password:(NSString*)password response:(FYCRequestQueueResponse)response;

/**
 添加model,向队列中添加任务

 @param task FYCRequestQueueTask中的url和params不能为空
 */
- (void)addWithTask:(FYCRequestQueueTask *)task;

/**
 更新参数,更新FYCRequestQueueTask的params

 @param taskId 用于查找FYCRequestQueueTask
 @param paramsDic 新增的参数
 */
- (void)updateWithTaskId:(NSString*)taskId paramsDic:(NSDictionary*)paramsDic;

/**
 删除任务

 @param taskId 任务Id
 */
- (void)removeWithTaskId:(NSString *)taskId;

/**
 执行任务
 如果队列中已经有任务在执行,则等待执行,如果没有则开始执行任务
 */
- (void)tick;

@end
