//
//  FYCResponse.h
//  FYClatformDemo
//
//  Created by huangsy on 2018/5/23.
//  Copyright © 2018年 stars. All rights reserved.
//

#import <Foundation/Foundation.h>

@class FYCResponse;

typedef NS_ENUM(NSInteger, FYCResponseStatus) {
    FYCResponseSuccess = 0, /// 成功
    FYCResponseFailure = -1 /// 失败
};


/**
 响应回调，回调一个 FYCResponse 实例

 @param resp FYCResponse 实例
 */
typedef void(^FYCResponseBlock)(FYCResponse *resp);


/**
 一个通用的响应类
 可以用在一些回调中，存放回调参数
 */
@interface FYCResponse : NSObject

/// 状态码
@property (nonatomic, assign) NSInteger status;
/// 描述信息
@property (nonatomic, copy) NSString *message;
/// 参数信息
@property (nonatomic, copy) NSDictionary *data;

/**
 初始化

 @param status 状态码
 @param message 描述信息
 @param data 参数信息
 @return instance
 */
- (instancetype)initWithStatus:(NSInteger)status message:(NSString*)message data:(NSDictionary*)data;

/**
 往 data 字典中设置 key 和 value

 @param key key
 @param value value
 */
- (void)setDataWithKey:(NSString*)key value:(id)value;

/**
 根据 key 从 data 字典中取出 value
 
 @param key key
 @return value
 */
- (id)getDataWithKey:(NSString*)key;

/**
 将实例格式化为字典格式

 @return dict
 */
- (NSDictionary*)toDict;

/**
 将实例格式化为JSON 格式

 @return JSON
 */
- (NSString*)toJSON;

@end
