//
//  HSYUserDefaults.h
//  QianHuoClientIOS
//
//  Created by huangsy on 16/3/30.
//  Copyright © 2016年 feiyu. All rights reserved.
//

#import <UIKit/UIKit.h>

/**
 对 NSUserDefault 的封装
 */
@interface FYCStorage : NSObject

/**
 注册一个默认值

 @param defaultValue 默认值
 @param key 对应的 key
 */
+ (void)registerDefaults:(id)defaultValue forKey:(NSString*)key;


/**
 保存对象

 @param object 对象
 @param key key
 */
+ (void)setObject:(id)object forKey:(NSString*)key;

/**
 获取对象

 @param key key
 @return  对象
 */
+ (id)objectForKey:(NSString*)key;

/**
 删除对象

 @param key key
 */
+ (void)removeObjectForKey:(NSString*)key;

/**
 保存 int

 @param integer int 值
 @param key key
 */
+ (void)setInteger:(NSInteger)integer forKey:(NSString*)key;

/**
 获取 int

 @param key key
 @return int 值
 */
+ (NSInteger)integerForKey:(NSString*)key;

/**
 保存 float

 @param sFloat float 值
 @param key key
 */
+ (void)setFloat:(CGFloat)sFloat forKey:(NSString*)key;

/**
 获取 float

 @param key key
 @return float 值
 */
+ (CGFloat)floatForKey:(NSString*)key;

/**
 保存bool

 @param b bool
 @param key key
 */
+ (void)setBool:(BOOL)b forKey:(NSString*)key;

/**
 获取 bool

 @param key key
 @return bool
 */
+ (BOOL)boolForKey:(NSString*)key;

/**
 保存字典

 @param dict 字典
 @param key  key
 */
+ (void)setDictionary:(NSDictionary*)dict forKey:(NSString*)key;

/**
 获取字典

 @param key  key
 @return 字典
 */
+ (NSDictionary*)dictionaryForKey:(NSString*)key;

/**
 保存数组

 @param array 数组
 @param key  key
 */
+ (void)setArray:(NSArray*)array forKey:(NSString*)key;

/**
 获取数组

 @param key key
 @return  数组
 */
+ (NSArray*)arrayForKey:(NSString*)key;

/**
 给保存在 user default 里的数组插入一个值

 @param object 要插入的值
 @param key  key
 */
+ (void)addValue:(id)object forArrayKey:(NSString*)key;

/**
 给保存在 user default 里的数组删除一个值

 @param object 要删除的值
 @param key key
 */
+ (void)removeValue:(id)object forArrayKey:(NSString*)key;

/**
 取出保存在 user default 里的数组的第一个值

 @param key key
 @return 数组的第一个值
 */
+ (id)firstValueForArrayKey:(NSString*)key;

/**
 判断保存在 user default 里的数组是否有值

 @param key key
 @return bool
 */
+ (BOOL)hasValueForArrayKey:(NSString*)key;

/**
 加密字符串并保存，采用 AES 对称加密，提高保存在 user default 中的数据的安全性

 @param string 要保存的字符串
 @param key key
 @param password 加密密码
 */
+ (void)setString:(NSString*)string forKey:(NSString*)key encryptWithPassword:(NSString*)password;

/**
 获取字符串并解密

 @param key key
 @param password 解密密码，与加密密码是统一个
 @return 解密后的字符串
 */
+ (NSString*)stringForKey:(NSString*)key decryptWithPassword:(NSString*)password;

/**
 加密字段并保存，采用 AES 对称加密，提高保存在 user default 中的数据的安全性

 @param dict 要保存的字典
 @param key key
 @param password 加密密码
 */
+ (void)setDictionary:(NSDictionary*)dict forKey:(NSString*)key encryptWithPassword:(NSString*)password;

/**
 获取字典并解密

 @param key key
 @param password 解密密码，与加密密码是统一个
 @return 解密后的字典
 */
+ (NSDictionary*)dictionaryForKey:(NSString*)key decryptWithPassword:(NSString*)password;

/**
 为数组添加成员

 @param value value
 @param key key
 */
+ (void)setArrayItem:(NSString*)value forKey:(NSString*)key;

/**
 获取数组中的成员

 @param key key
 @param index 索引
 @return return value
 */
+ (id)arrayItemForKey:(NSString*)key atIndex:(NSInteger)index;

/**
 根据索引删除数组成员

 @param key key
 @param index 索引
 */
+ (void)removeArrayItemForKey:(NSString*)key atIndex:(NSInteger)index;

/**
 根据成员删除数组成员

 @param key key
 @param value value
 */
+ (void)removeArrayItemForKey:(NSString*)key value:(NSString*)value;

@end
