//
//  FYCURLSession.h
//
//  Created by huangsy on 18/11/10.
//  Copyright © 2018年 huangsy. All rights reserved.
//

#import <Foundation/Foundation.h>

@class FYCURLSession;

@protocol FYCURLSessionDelegate

/**
 请求回调方法

 @param connected 是否连接服务器成功
 @param resp 服务器返回的字符串
 @param httpInfo http的请求信息 和 http的响应信息
 */
- (void)fycURLSessionConnected:(BOOL)connected resp:(NSString*)resp httpInfo:(NSDictionary*)httpInfo DEPRECATED_MSG_ATTRIBUTE("10266弃用");

@end

/**
 请求回调 block，回调在主线程

 @param connected 是否连接服务器成功
 @param resp 服务器返回的字符串
 @param httpInfo http的请求信息 和 http的响应信息
 */
typedef void(^FYCURLSessionResponse)(BOOL connected, NSString *resp, NSDictionary *httpInfo) DEPRECATED_MSG_ATTRIBUTE("10266弃用");

/**
请求回调 block，回调在主线程

@param error 错误信息
@param response 服务器返回的字符串
*/
typedef void(^FYCRequestResponse)(NSError *error, NSString *response) DEPRECATED_MSG_ATTRIBUTE("10266弃用");

/// 请求完成
/// responseData 响应数据
/// error 请求错误信息
/// request 请求信息
/// response 响应信息
typedef void(^FYCRequestComplete) (NSString *responseData, NSError *error, NSURLRequest *request, NSHTTPURLResponse *response);

/**
 自定义网络请求类，用来做简单请求
 多个相似的请求可以共用一个实例
 */
@interface FYCURLSession : NSObject

/// 设置超时时间， 默认 10秒
@property (nonatomic, assign) NSTimeInterval timeout;
/// 设置请求头，FYCURLSession生命周期内都会使用
@property (nonatomic, copy) NSDictionary<NSString*, NSString*> *headers;

/// 开始一个请求
/// urlStr 请求地址
/// params 请求参数
/// method 请求方式 GET、POST
/// headers 单次请求的请求头
/// completion 请求完成回调
- (void)request:(NSString*)urlStr params:(NSDictionary*)params method:(NSString*)method headers:(NSDictionary*)headers completion:(FYCRequestComplete)completion;


#pragma mark - 过期方法

/// 设置请求方式，GET/POST，default GET
@property (nonatomic, copy) NSString *httpMethod DEPRECATED_MSG_ATTRIBUTE("10266弃用");
/// 设置回调代理
@property (nonatomic, weak) id<FYCURLSessionDelegate> delegate DEPRECATED_MSG_ATTRIBUTE("10266弃用");
/// 设置是否允许缓存，default NO
@property (nonatomic, assign) BOOL isCache DEPRECATED_MSG_ATTRIBUTE("10266弃用");
/**
 初始化

 @param httpMethod 请求方式，GET/POST
 @return instance
 */
- (instancetype)initWithHTTPMethod:(NSString*)httpMethod DEPRECATED_MSG_ATTRIBUTE("10266弃用");
/**
 开启一个请求，建议使用- (void)fyc_requestWithURL:params:response:，后续将逐渐废弃 requestWithURL:

 @param urlStr 请求地址
 @param paramsDict 请求参数
 @param responseBlock 请求回调
 */
- (void)requestWithURL:(NSString*)urlStr params:(NSDictionary*)paramsDict response:(FYCURLSessionResponse)responseBlock DEPRECATED_MSG_ATTRIBUTE("10266弃用");
/// 发起数据请求
/// @param urlStr 请求地址
/// @param params 请求参数
/// @param completion 请求回调
- (void)fyc_requestWithURL:(NSString*)urlStr params:(NSDictionary*)params completion:(FYCRequestResponse)completion DEPRECATED_MSG_ATTRIBUTE("10266弃用");
/**
 开启一个请求

 @param urlStr 请求地址
 @param paramsDict 请求参数
 */
- (void)requestWithURL:(NSString*)urlStr params:(NSDictionary*)paramsDict DEPRECATED_MSG_ATTRIBUTE("10266弃用");

@end
