//
//  NBUtils.h
//
//  Created by huangsy on 15-1-23.
//  Copyright (c) 2015年 feiyu.com. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AdSupport/ASIdentifierManager.h>
#import <UIKit/UIKit.h>


/**
 常用工具类
 */
@interface FYCUtils : NSObject

/// 获取当前屏幕显示的 view controller
+ (UIViewController *)currentViewController;

/// 获取最上层 window
+ (UIView*)topWindow;

/// URL解码
+ (NSString*)urlDecode:(NSString*)string;

/// URL编码
+ (NSString*)urlEncode:(NSString*)string;

/// URL参数字符串解析成dictionary
+ (NSDictionary*)dictWithURLParams:(NSString*)string;

/// dictionary转成URL参数字符串
+ (NSString*)urlParamsWithDict:(NSDictionary*)dict;

/// 1.dictionary按key排序 2.组合URL参数字符串 3.URL编码 4.生成签名
+ (NSString*)signWithDict:(NSDictionary*)dict secret:(NSString *)secret;

/// rsa 签名
+ (NSString*)rsaSignWithDict:(NSDictionary*)dict publicKey:(NSString*)publicKey;

/// MD5
+ (NSString*)md5:(NSString*)string;

/// 屏幕截图，并保存在相册
+ (void)screenShot;

/// JSON解析成NSDictionary
+ (NSDictionary*)dictWithJSON:(NSString*)jsonString;

/// Dictionary解析成JSON
+ (NSString*)JSONWithDict:(NSDictionary*)dict;

/// JSON解析成Array
+ (NSArray*)arrayWithJSON:(NSString*)jsonString;

/// Array解析成JSON
+ (NSString*)JSONWithArray:(NSArray*)arr;

/// 获取时间戳（秒）
+ (NSString*)timeStamp;

// 获取时间戳（毫秒）
+ (NSString*)msecTimeStamp;

/// 根据 Formatter 获取当前系统时间
+ (NSString*)systemTimeWithFormatter:(NSString *)formatterString;

/// 根据 Formatter 获取系统时间
+ (NSString*)systemTimeWithDate:(NSDate*)date formatter:(NSString *)formatterString;

/// 根据 timeStamp、formatter 将时间戳转为时间
+ (NSString *)timeStampToTime:(NSString *)timeStamp formatter:(NSString *)formatterString;

/// 获取系统时间(yyyyMMddHHmmss)
+ (NSString*)systemTime;

/// 获取系统时间(HH:mm:ss)
+ (NSString*)systemTime1;

//获取系统时间(HH:mm:ss.SSS)
+ (NSString*)systemTime2;

/// 获取年月日(xxxx年xx月xx日)
+ (NSString*)stringDateWithDate:(NSDate*)date;

/// 获取时分秒(xx时xx分xx秒)
+ (NSString*)stringTimeWithDate:(NSDate*)date;

/// 获取当前 年
+ (NSInteger)year;

/// 获取当前 月
+ (NSInteger)month;

/// 获取当前 日
+ (NSInteger)day;

/// 获取当前 时
+ (NSInteger)hour;

/// 获取当前 分
+ (NSInteger)minute;

/// 获取当前 秒
+ (NSInteger)second;

/// 数组排序
+ (NSArray*)sortArray:(NSArray*)array;

/// 格式验证：邮箱、电话
+ (BOOL)validateEmailAndTel:(NSString*)string;

/// 格式验证：电话
+ (BOOL)validateTel:(NSString*)tel;

/// 格式验证：邮箱
+ (BOOL)validateEmail:(NSString*)email;

/// 格式验证：真实姓名
+ (BOOL)validateRealName:(NSString*)realName;

/// 格式验证：身份证号码
+ (BOOL)validateIdNumber:(NSString*)idNumber;

/// 生成随机整数 包含from 包含to
+ (NSInteger)randomNumberFrom:(NSInteger)from to:(NSInteger)to;

/// 在from 到 to 中， 取count个随机整数
+ (NSArray*)randomNumbersForm:(NSInteger)from to:(NSInteger)to count:(NSInteger)count;

/// 生成随机数id
+ (NSString*)createNumberId;

/// 去除字符串空格
+ (NSString*)removeWhiteSpace:(NSString*)str;

/// 安全号码
+ (NSString*)safePhone:(NSString*)phone;

/// 省略文字
+ (NSString*)stringEllipsis:(NSString*)str retain:(NSInteger)retain;

/// 获取时精确间戳
+ (NSString*)accurateTimeStamp;

/// 获取随机字符串
+ (NSString *)randomString:(NSInteger)count;

@end
