//
//  FYMASConstraint+Private.h
//  Masonry
//
//  Created by Nick Tymchenko on 29/04/14.
//  Copyright (c) 2014 cloudling. All rights reserved.
//

#import "FYMASConstraint.h"

@protocol FYMASConstraintDelegate;


@interface FYMASConstraint ()

/**
 *  Whether or not to check for an existing constraint instead of adding constraint
 */
@property (nonatomic, assign) BOOL updateExisting;

/**
 *	Usually FYMASConstraintMaker but could be a parent FYMASConstraint
 */
@property (nonatomic, weak) id<FYMASConstraintDelegate> delegate;

/**
 *  Based on a provided value type, is equal to calling:
 *  NSNumber - setOffset:
 *  NSValue with CGPoint - setPointOffset:
 *  NSValue with CGSize - setSizeOffset:
 *  NSValue with FYMASEdgeInsets - setInsets:
 */
- (void)setLayoutConstantWithValue:(NSValue *)value;

@end


@interface FYMASConstraint (Abstract)

/**
 *	Sets the constraint relation to given NSLayoutRelation
 *  returns a block which accepts one of the following:
 *    FYMASViewAttribute, UIView, NSValue, NSArray
 *  see readme for more details.
 */
- (FYMASConstraint * (^)(id, NSLayoutRelation))equalToWithRelation;

/**
 *	Override to set a custom chaining behaviour
 */
- (FYMASConstraint *)addConstraintWithLayoutAttribute:(NSLayoutAttribute)layoutAttribute;

@end


@protocol FYMASConstraintDelegate <NSObject>

/**
 *	Notifies the delegate when the constraint needs to be replaced with another constraint. For example
 *  A FYMASViewConstraint may turn into a FYMASCompositeConstraint when an array is passed to one of the equality blocks
 */
- (void)constraint:(FYMASConstraint *)constraint shouldBeReplacedWithConstraint:(FYMASConstraint *)replacementConstraint;

- (FYMASConstraint *)constraint:(FYMASConstraint *)constraint addConstraintWithLayoutAttribute:(NSLayoutAttribute)layoutAttribute;

@end
