//
//  FYMASViewConstraint.h
//  Masonry
//
//  Created by Jonas Budelmann on 20/07/13.
//  Copyright (c) 2013 cloudling. All rights reserved.
//

#import "FYMASViewAttribute.h"
#import "FYMASConstraint.h"
#import "FYMASLayoutConstraint.h"
#import "FYMASUtilities.h"

/**
 *  A single constraint.
 *  Contains the attributes neccessary for creating a NSLayoutConstraint and adding it to the appropriate view
 */
@interface FYMASViewConstraint : FYMASConstraint <NSCopying>

/**
 *	First item/view and first attribute of the NSLayoutConstraint
 */
@property (nonatomic, strong, readonly) FYMASViewAttribute *firstViewAttribute;

/**
 *	Second item/view and second attribute of the NSLayoutConstraint
 */
@property (nonatomic, strong, readonly) FYMASViewAttribute *secondViewAttribute;

/**
 *	initialises the FYMASViewConstraint with the first part of the equation
 *
 *	@param	firstViewAttribute	view.mas_left, view.mas_width etc.
 *
 *	@return	a new view constraint
 */
- (id)initWithFirstViewAttribute:(FYMASViewAttribute *)firstViewAttribute;

/**
 *  Returns all FYMASViewConstraints installed with this view as a first item.
 *
 *  @param  view  A view to retrieve constraints for.
 *
 *  @return An array of FYMASViewConstraints.
 */
+ (NSArray *)installedConstraintsForView:(FYMAS_VIEW *)view;

@end
