//
//  UIView+FYMASAdditions.h
//  Masonry
//
//  Created by Jonas Budelmann on 20/07/13.
//  Copyright (c) 2013 cloudling. All rights reserved.
//

#import "FYMASUtilities.h"
#import "FYMASConstraintMaker.h"
#import "FYMASViewAttribute.h"

/**
 *	Provides constraint maker block
 *  and convience methods for creating FYMASViewAttribute which are view + NSLayoutAttribute pairs
 */
@interface FYMAS_VIEW (FYMASAdditions)

/**
 *	following properties return a new FYMASViewAttribute with current view and appropriate NSLayoutAttribute
 */
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_left;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_top;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_right;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_bottom;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_leading;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_trailing;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_width;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_height;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_centerX;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_centerY;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_baseline;
@property (nonatomic, strong, readonly) FYMASViewAttribute *(^mas_attribute)(NSLayoutAttribute attr);

@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_firstBaseline;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_lastBaseline;

#if TARGET_OS_IPHONE || TARGET_OS_TV

@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_leftMargin;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_rightMargin;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_topMargin;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_bottomMargin;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_leadingMargin;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_trailingMargin;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_centerXWithinMargins;
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_centerYWithinMargins;

@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_safeAreaLayoutGuide NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_safeAreaLayoutGuideLeading NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_safeAreaLayoutGuideTrailing NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_safeAreaLayoutGuideLeft NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_safeAreaLayoutGuideRight NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_safeAreaLayoutGuideTop NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_safeAreaLayoutGuideBottom NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_safeAreaLayoutGuideWidth NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_safeAreaLayoutGuideHeight NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_safeAreaLayoutGuideCenterX NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *mas_safeAreaLayoutGuideCenterY NS_AVAILABLE_IOS(11.0);

#endif

/**
 *	a key to associate with this view
 */
@property (nonatomic, strong) id mas_key;

/**
 *	Finds the closest common superview between this view and another view
 *
 *	@param	view	other view
 *
 *	@return	returns nil if common superview could not be found
 */
- (instancetype)mas_closestCommonSuperview:(FYMAS_VIEW *)view;

/**
 *  Creates a FYMASConstraintMaker with the callee view.
 *  Any constraints defined are added to the view or the appropriate superview once the block has finished executing
 *
 *  @param block scope within which you can build up the constraints which you wish to apply to the view.
 *
 *  @return Array of created FYMASConstraints
 */
- (NSArray *)mas_makeConstraints:(void(NS_NOESCAPE ^)(FYMASConstraintMaker *make))block;

/**
 *  Creates a FYMASConstraintMaker with the callee view.
 *  Any constraints defined are added to the view or the appropriate superview once the block has finished executing.
 *  If an existing constraint exists then it will be updated instead.
 *
 *  @param block scope within which you can build up the constraints which you wish to apply to the view.
 *
 *  @return Array of created/updated FYMASConstraints
 */
- (NSArray *)mas_updateConstraints:(void(NS_NOESCAPE ^)(FYMASConstraintMaker *make))block;

/**
 *  Creates a FYMASConstraintMaker with the callee view.
 *  Any constraints defined are added to the view or the appropriate superview once the block has finished executing.
 *  All constraints previously installed for the view will be removed.
 *
 *  @param block scope within which you can build up the constraints which you wish to apply to the view.
 *
 *  @return Array of created/updated FYMASConstraints
 */
- (NSArray *)mas_remakeConstraints:(void(NS_NOESCAPE ^)(FYMASConstraintMaker *make))block;

@end
