//
//  UIView+FYMASShorthandAdditions.h
//  Masonry
//
//  Created by Jonas Budelmann on 22/07/13.
//  Copyright (c) 2013 Jonas Budelmann. All rights reserved.
//

#import "View+FYMASAdditions.h"

#ifdef FYMAS_SHORTHAND

/**
 *	Shorthand view additions without the 'mas_' prefixes,
 *  only enabled if FYMAS_SHORTHAND is defined
 */
@interface FYMAS_VIEW (FYMASShorthandAdditions)

@property (nonatomic, strong, readonly) FYMASViewAttribute *left;
@property (nonatomic, strong, readonly) FYMASViewAttribute *top;
@property (nonatomic, strong, readonly) FYMASViewAttribute *right;
@property (nonatomic, strong, readonly) FYMASViewAttribute *bottom;
@property (nonatomic, strong, readonly) FYMASViewAttribute *leading;
@property (nonatomic, strong, readonly) FYMASViewAttribute *trailing;
@property (nonatomic, strong, readonly) FYMASViewAttribute *width;
@property (nonatomic, strong, readonly) FYMASViewAttribute *height;
@property (nonatomic, strong, readonly) FYMASViewAttribute *centerX;
@property (nonatomic, strong, readonly) FYMASViewAttribute *centerY;
@property (nonatomic, strong, readonly) FYMASViewAttribute *baseline;
@property (nonatomic, strong, readonly) FYMASViewAttribute *(^attribute)(NSLayoutAttribute attr);

@property (nonatomic, strong, readonly) FYMASViewAttribute *firstBaseline;
@property (nonatomic, strong, readonly) FYMASViewAttribute *lastBaseline;

#if TARGET_OS_IPHONE || TARGET_OS_TV

@property (nonatomic, strong, readonly) FYMASViewAttribute *leftMargin;
@property (nonatomic, strong, readonly) FYMASViewAttribute *rightMargin;
@property (nonatomic, strong, readonly) FYMASViewAttribute *topMargin;
@property (nonatomic, strong, readonly) FYMASViewAttribute *bottomMargin;
@property (nonatomic, strong, readonly) FYMASViewAttribute *leadingMargin;
@property (nonatomic, strong, readonly) FYMASViewAttribute *trailingMargin;
@property (nonatomic, strong, readonly) FYMASViewAttribute *centerXWithinMargins;
@property (nonatomic, strong, readonly) FYMASViewAttribute *centerYWithinMargins;

#endif

#if TARGET_OS_IPHONE || TARGET_OS_TV

@property (nonatomic, strong, readonly) FYMASViewAttribute *safeAreaLayoutGuideLeading NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *safeAreaLayoutGuideTrailing NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *safeAreaLayoutGuideLeft NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *safeAreaLayoutGuideRight NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *safeAreaLayoutGuideTop NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *safeAreaLayoutGuideBottom NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *safeAreaLayoutGuideWidth NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *safeAreaLayoutGuideHeight NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *safeAreaLayoutGuideCenterX NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) FYMASViewAttribute *safeAreaLayoutGuideCenterY NS_AVAILABLE_IOS(11.0);

#endif

- (NSArray *)makeConstraints:(void(^)(FYMASConstraintMaker *make))block;
- (NSArray *)updateConstraints:(void(^)(FYMASConstraintMaker *make))block;
- (NSArray *)remakeConstraints:(void(^)(FYMASConstraintMaker *make))block;

@end

#define FYMAS_ATTR_FORWARD(attr)  \
- (FYMASViewAttribute *)attr {    \
    return [self mas_##attr];   \
}

#define FYMAS_ATTR_FORWARD_AVAILABLE(attr, available)  \
- (FYMASViewAttribute *)attr available {    \
    return [self mas_##attr];   \
}

@implementation FYMAS_VIEW (FYMASShorthandAdditions)

FYMAS_ATTR_FORWARD(top);
FYMAS_ATTR_FORWARD(left);
FYMAS_ATTR_FORWARD(bottom);
FYMAS_ATTR_FORWARD(right);
FYMAS_ATTR_FORWARD(leading);
FYMAS_ATTR_FORWARD(trailing);
FYMAS_ATTR_FORWARD(width);
FYMAS_ATTR_FORWARD(height);
FYMAS_ATTR_FORWARD(centerX);
FYMAS_ATTR_FORWARD(centerY);
FYMAS_ATTR_FORWARD(baseline);

FYMAS_ATTR_FORWARD(firstBaseline);
FYMAS_ATTR_FORWARD(lastBaseline);

#if TARGET_OS_IPHONE || TARGET_OS_TV

FYMAS_ATTR_FORWARD(leftMargin);
FYMAS_ATTR_FORWARD(rightMargin);
FYMAS_ATTR_FORWARD(topMargin);
FYMAS_ATTR_FORWARD(bottomMargin);
FYMAS_ATTR_FORWARD(leadingMargin);
FYMAS_ATTR_FORWARD(trailingMargin);
FYMAS_ATTR_FORWARD(centerXWithinMargins);
FYMAS_ATTR_FORWARD(centerYWithinMargins);

FYMAS_ATTR_FORWARD_AVAILABLE(safeAreaLayoutGuideLeading, NS_AVAILABLE_IOS(11.0));
FYMAS_ATTR_FORWARD_AVAILABLE(safeAreaLayoutGuideTrailing, NS_AVAILABLE_IOS(11.0));
FYMAS_ATTR_FORWARD_AVAILABLE(safeAreaLayoutGuideLeft, NS_AVAILABLE_IOS(11.0));
FYMAS_ATTR_FORWARD_AVAILABLE(safeAreaLayoutGuideRight, NS_AVAILABLE_IOS(11.0));
FYMAS_ATTR_FORWARD_AVAILABLE(safeAreaLayoutGuideTop, NS_AVAILABLE_IOS(11.0));
FYMAS_ATTR_FORWARD_AVAILABLE(safeAreaLayoutGuideBottom, NS_AVAILABLE_IOS(11.0));
FYMAS_ATTR_FORWARD_AVAILABLE(safeAreaLayoutGuideWidth, NS_AVAILABLE_IOS(11.0));
FYMAS_ATTR_FORWARD_AVAILABLE(safeAreaLayoutGuideHeight, NS_AVAILABLE_IOS(11.0));
FYMAS_ATTR_FORWARD_AVAILABLE(safeAreaLayoutGuideCenterX, NS_AVAILABLE_IOS(11.0));
FYMAS_ATTR_FORWARD_AVAILABLE(safeAreaLayoutGuideCenterY, NS_AVAILABLE_IOS(11.0));

#endif

- (FYMASViewAttribute *(^)(NSLayoutAttribute))attribute {
    return [self mas_attribute];
}

- (NSArray *)makeConstraints:(void(NS_NOESCAPE ^)(FYMASConstraintMaker *))block {
    return [self mas_makeConstraints:block];
}

- (NSArray *)updateConstraints:(void(NS_NOESCAPE ^)(FYMASConstraintMaker *))block {
    return [self mas_updateConstraints:block];
}

- (NSArray *)remakeConstraints:(void(NS_NOESCAPE ^)(FYMASConstraintMaker *))block {
    return [self mas_remakeConstraints:block];
}

@end

#endif
