/*
 * Tencent is pleased to support the open source community by making
 * FYKV available.
 *
 * Copyright (C) 2018 THL A29 Limited, a Tencent company.
 * All rights reserved.
 *
 * Licensed under the BSD 3-Clause License (the "License"); you may not use
 * this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *       https://opensource.org/licenses/BSD-3-Clause
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FYKVHandler_h
#define FYKVHandler_h
#import <Foundation/Foundation.h>

typedef NS_ENUM(NSUInteger, FYKVRecoverStrategic) {
    FYKVOnErrorDiscard = 0,
    FYKVOnErrorRecover,
};

typedef NS_ENUM(NSUInteger, FYKVLogLevel) {
    FYKVLogDebug = 0, // not available for release/product build
    FYKVLogInfo = 1,  // default level
    FYKVLogWarning,
    FYKVLogError,
    FYKVLogNone, // special level used to disable all log messages
};

// callback is called on the operating thread of the FYKV instance
@protocol FYKVHandler <NSObject>
@optional

// by default FYKV will discard all datas on crc32-check failure
// return `FYKVOnErrorRecover` to recover any data on the file
- (FYKVRecoverStrategic)onFYKVCRCCheckFail:(NSString *)mmapID;

// by default FYKV will discard all datas on file length mismatch
// return `FYKVOnErrorRecover` to recover any data on the file
- (FYKVRecoverStrategic)onFYKVFileLengthError:(NSString *)mmapID;

// by default FYKV will print log using NSLog
// implement this method to redirect FYKV's log
- (void)fykvLogWithLevel:(FYKVLogLevel)level file:(const char *)file line:(int)line func:(const char *)funcname message:(NSString *)message;

// called when content is changed by other process
// doesn't guarantee real-time notification
- (void)onFYKVContentChange:(NSString *)mmapID;

@end

#endif /* FYKVHandler_h */
